#    Highly extensible Dependency injection framework for humans
#    Copyright (C) 2017 Dmitriy Selischev
#    The MIT License (MIT)
#    
#    Permission is hereby granted, free of charge, to any person obtaining
#    a copy of this software and associated documentation files
#    (the "Software"), to deal in the Software without restriction,
#    including without limitation the rights to use, copy, modify, merge,
#    publish, distribute, sublicense, and/or sell copies of the Software,
#    and to permit persons to whom the Software is furnished to do so,
#    subject to the following conditions:
#    
#    The above copyright notice and this permission notice shall be
#    included in all copies or substantial portions of the Software.
#    
#    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
#    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
#    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
#    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
#    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import importlib
import typing
from hexdi import utils


class AbstractBaseLoader(object):
    def load(self):
        pass


class AbstractOneTimeLoader(AbstractBaseLoader):
    def __init__(self):
        self.__loaded = False

    def load(self):
        """
        load classes to providing auto register
        """
        if not self.__loaded:
            self._do_loading()
            self.__loaded = True

    def _do_loading(self):
        pass


class BasicLoader(AbstractOneTimeLoader):
    def __init__(self, modules: typing.Iterable):
        super().__init__()
        self.__modules = modules

    def _do_loading(self):
        for mod in self.__modules:
            utils.load_module(utils.get_module_name(mod))
